<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use BadFunctionCallException;
use BadMethodCallException;
use Infomaker\Everyware\Support\Str;

/**
 * Class DefaultTemplateFunctions
 * @package Infomaker\Everyware\Twig
 */
class DefaultTemplateFunctions
{
    public function inProduction(): bool
    {
        return is_prod();
    }

    public function phpFunction($name, $args = [])
    {
        if ( ! function_exists($name)) {
            throw new BadFunctionCallException('Function dose not exist');
        }

        return call_user_func_array($name, $args);
    }

    public function phpMethod($name, $args = [])
    {
        $staticMethod = strpos($name, '::') ? explode('::', $name) : Str::parseCallback($name, '');

        if ( ! method_exists($staticMethod[0], $staticMethod[1])) {
            throw new BadMethodCallException('Method ' . $staticMethod[1] . ' can not be found on ' . $staticMethod[0]);
        }

        return call_user_func_array(implode('::', $staticMethod), $args);
    }

    public function renderClasses($classes): string
    {
        return implode(' ', array_filter((array)$classes));
    }

    public function renderPartial(string $template, array $data = []): void
    {
        View::render($template, $data);
    }

}
