<?php

namespace Infomaker\Everyware\Twig\Tests\Features\Template;

use PHPUnit\Framework\TestCase;
use PHPUnit\Framework\MockObject\Generator;
use Behat\Behat\Context\SnippetAcceptingContext;
use Infomaker\Everyware\Twig\View;
use Infomaker\Everyware\Twig\Tests\Features\Traits\ViewSetupCommon;
use Infomaker\Everyware\Twig\Tests\src\TwigFunctions;
use Infomaker\Everyware\Support\Str;


/**
 * This is a behat test class, using Gherkin descriptions.
 * Further explanation of what is being tested and method descriptions can be found in the .feature files.
 *
 * --!! IMPORTANT DO NOT CHANGE THE DOC BLOCKS !!--
 */
class TemplateContext implements SnippetAcceptingContext
{
    use ViewSetupCommon;

    private $functions = [];
    
    /**
     * @Then PHP functions can be added
     */
    public function php_functions_can_be_added_to_the_view()
    {
        $twigFunctions = new TwigFunctions();
        foreach (\get_class_methods($twigFunctions) as $funcName) {
            $this->viewSetup->addFunction(Str::snake($funcName), [$twigFunctions, $funcName]);
        }
    }

    /**
     * @Then the twig template :template can use PHP functions
     */
    public function the_template_can_use_php_functions($template)
    {
        $view = View::generate('@namespaceTest/' . $template);
        TestCase::assertNotNull($view);
        TestCase::assertContains('Does this work', $view);
        TestCase::assertContains('Sure hope so!', $view);
        TestCase::assertInternalType('string', $view);
    }

    /**
     * @Then Globals can be added :key :value
     */
    public function globals_can_be_added_to_twig(string $key, $value)
    {
        $this->viewSetup->addGlobal($key, $value);
    }

    /**
     * @Then the twig template :template can use Globals
     */
    public function the_twig_template_can_use_globals($template)
    {
        $view = View::generate('@namespaceTest/' . $template);
        TestCase::assertNotNull($view);
        TestCase::assertContains('Does', $view);
        TestCase::assertContains('This', $view);
        TestCase::assertContains('Work', $view);
        TestCase::assertInternalType('string', $view);
    }

    /**
     * @Then filters can be added :filter
     */
    public function filters_can_be_added($filter)
    {
        $callable = function ($string) {
            return strtoupper($string);
        };

        $this->viewSetup->addFilter($filter, $callable);
    }

    /**
     * @Then the twig template :template can use filters
     */
    public function the_twig_template_can_use_filters($template)
    {
        $view = View::generate('@namespaceTest/' . $template);
        TestCase::assertNotNull($view);
        TestCase::assertContains('DOES', $view);
        TestCase::assertContains('THIS', $view);
        TestCase::assertContains('WORK', $view);
        TestCase::assertInternalType('string', $view);
    }

    /**
     * @Then the twig template :template can use filters, globals and PHP functions
     */
    public function the_twig_template_can_use_filters_globals_and_functions($template)
    {
        $view = View::generate('@namespaceTest/' . $template);
        TestCase::assertNotNull($view);
        TestCase::assertContains('DOES', $view);
        TestCase::assertContains('THIS', $view);
        TestCase::assertContains('WORK', $view);
        TestCase::assertContains('Uno', $view);
        TestCase::assertContains('Dos', $view);
        TestCase::assertContains('Tres', $view);
        TestCase::assertContains('Does this work', $view);
        TestCase::assertContains('Sure hope so!', $view);
        TestCase::assertInternalType('string', $view);
    }
}
