<?php
namespace Infomaker\Everyware\Twig\Tests\Features\Traits;

use PHPUnit\Framework\MockObject\Generator;
use Behat\Behat\Context\SnippetAcceptingContext;
use Infomaker\Everyware\Twig\EveryTwig;
use Infomaker\Everyware\Twig\FilesystemLoader;
use Infomaker\Everyware\Twig\ViewSetup;
use Behat\Behat\Hook\Scope\BeforeScenarioScope;

trait ViewSetupCommon
{
    private $viewSetup;

    /** @BeforeScenario */
    public function before(BeforeScenarioScope $scope)
    {
        $instance = ViewSetup::getInstance();
        $reflection = new \ReflectionClass($instance);
        $instance = $reflection->getProperty('instance');
        $instance->setAccessible(true); // now we can modify that :)
        $instance->setValue(null, null); // instance is gone
        $instance->setAccessible(false); // clean up
    }

    /**
     * @Given viewsetup can be initiated 
     */
    public function viewSetup_can_be_initiated()
    {
        $this->viewSetup = ViewSetup::getInstance();
    }
    /**
     * @Then a loader can be set
     */
    public function a_loader_can_be_set()
    {
        $methods = [
            'getCache',
            'getPathCacheKey',
            'setCache'
        ];
        $this->viewSetup->setLoader((new Generator())->getMock(FilesystemLoader::class, $methods));
    }

    /**
     * @Then EveryTwig can be set
     */
    public function everyTwig_can_be_set()
    {
        $this->viewSetup->setEveryTwig(EveryTwig::getInstance());
    }

    /**
     * @Then the folder path :path and :namespace can be added
     */
    public function the_folder_path_and_namespace_can_be_added($path, $namespace)
    {
        $this->viewSetup->registerTwigFolder($namespace, __DIR__ . '/../../' . $path);
        
    }
    /**
     * @Then I can init EveryTwig from ViewSetup
     */
    public function everyTwig_can_be_initiated_from_viewSetup()
    {
        $this->viewSetup->init();
    }
}
