<?php
declare(strict_types=1);

use PHPUnit\Framework\TestCase;
use Infomaker\Everyware\Twig\ViewSetup;
use Infomaker\Everyware\Twig\EveryTwig;
/**
 * ViewSetupTest
 * @group ViewSetup
 */
class ViewSetupTest extends TestCase
{
    private $viewSetup;

    public function setUp()
    {
        parent::setUp();

        // now recreate a fresh object
        $this->viewSetup = ViewSetup::getInstance();
    }
    protected function tearDown()
    {
        $instance = ViewSetup::getInstance();
        $reflection = new ReflectionClass($instance);
        $instance = $reflection->getProperty('instance');
        $instance->setAccessible(true); // now we can modify that :)
        $instance->setValue(null, null); // instance is gone
        $instance->setAccessible(false); // clean up

        $instance = EveryTwig::getInstance();
        $reflection = new ReflectionClass($instance);
        $instance = $reflection->getProperty('instance');
        $instance->setAccessible(true); // now we can modify that :)
        $instance->setValue(null, null); // instance is gone
        $instance->setAccessible(false); // clean up
    }
    /** @test */
    public function a_instance_of_everyTwig_can_be_set_and_fetched()
    {
        $this->viewSetup->setEveryTwig(EveryTwig::getInstance());
        $this->assertInstanceOf(EveryTwig::class, $this->viewSetup->getEveryTwig());
    }
    /** @test */
    public function a_view_can_add_wp_functions()
    {
        $instance = $this->viewSetup->addWpFunction('functionName', 'customNamed');
        $this->assertInstanceOf(ViewSetup::class, $instance);
        $instance = $this->viewSetup->addWpFunction('functionName');
        $this->assertInstanceOf(ViewSetup::class, $instance);
    }
    /** @test */
    public function a_view_can_only_add_a_wp_function_with_atleast_one_argument()
    {
        $this->expectException(ArgumentCountError::class);
        $this->viewSetup->addWpFunction();
    }
    /** @test */
    public function a_view_can_only_add_a_function_with_atleast_two_argument()
    {
        $this->expectException(ArgumentCountError::class);
        $this->viewSetup->addFunction();
    }
    /** @test */
    public function a_function_can_be_added_to_the_view()
    {
        $options = [
            'needs_environment' => true,
            'needs_context' => true,
            'is_variadic' => true,
            'is_safe' => null,
            'is_safe_callback' => null,
            'node_class' => 'Twig_Node_Expression_Function',
            'deprecated' => true,
            'alternative' => null,
        ];

        $instance = $this->viewSetup->addFunction('functionName', 'callable', $options);
        $this->assertInstanceOf(ViewSetup::class, $instance);
    }

    /** @test */
    public function a_static_instance_can_be_returned()
    {
        $this->assertInstanceOf(ViewSetup::class, ViewSetup::getInstance());
    }
    /** @test */
    public function a_twig_folder_can_be_registered()
    {
        $exclude = [
            '/root',
            '/admin'
        ];
        $instance = $this->viewSetup->registerTwigFolder('namespace', '/path', $exclude);
        $this->assertInstanceOf(ViewSetup::class, $instance);
    }
    /** @test */
    public function a_filter_can_be_added()
    {
        $options = [
            'needs_environment' => true,
            'needs_context' => true,
            'is_variadic' => true,
            'is_safe' => null,
            'is_safe_callback' => null,
            'pre_escape' => null,
            'preserves_safety' => null,
            'node_class' => 'Twig_Node_Expression_Filter',
            'deprecated' => true,
            'alternative' => null,
        ];
        $instance = $this->viewSetup->addFilter('filter', 'callable', $options);
        $this->assertInstanceOf(ViewSetup::class, $instance);
    }
    /** @test */
    public function a_global_can_be_added()
    {
        $instance = $this->viewSetup->addGlobal('key', 'value');
        $this->assertInstanceOf(ViewSetup::class, $instance);
    }
}
